# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment, Segment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimMultiple as HL7Multiple
from driverng.astm import HprimString as HL7String
from driverng.message import BinGraphContainer

logger = logging.getLogger("driverng.scopio_x100")


class ScopioGraphContainer(BinGraphContainer):
    def __init__(self, data=None):
        super(ScopioGraphContainer, self).__init__()
        self["data"] = data
        self["format"] = "jpeg"
        self["graph_type"] = "binary"


class ScopioX100SegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    message_type = HL7String((9, 1))
    message_code = HL7String((9, 2))
    control_id = HL7String(10)
    processing_id = HL7String(11)
    version_id = HL7String(12)

    patients = HL7Foreign("PID", "header")


class ScopioX100SegmentPID(Segment):
    patient_id = HL7String(4)
    patient_name = HL7String(6)

    visits = HL7Foreign("PV1", "patient")


class ScopioX100SegmentPV1(Segment):
    patient_class = HL7String(3)

    orders = HL7Foreign("OBR", "patient")


class ScopioX100SegmentOBR(Segment):
    placer_order_number = HL7String(3)
    filler_order_number = HL7String(4)
    universal_service_identifier = HL7String(5)
    observation_date = HL7Date(8)

    results = HL7Foreign("OBX", "order")


class ScopioX100SegmentOBX(Segment):
    MAX_VALUE_SUBFIELD = 5
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    coding_system = HL7String((4, 3))
    value = HL7String(6)
    embended_file = HL7Multiple(
        6,
        **dict(
            ("line{}".format(i), HL7String(i)) for i in range(1, MAX_VALUE_SUBFIELD + 1)
        )
    )
    units = HL7String(7)
    result_status = HL7String(12)
    operator_first_name = HL7String((17, 2))
    operator_last_name = HL7String((17, 3))
    operator_login = HL7String((17, 4))

    comments = HL7Foreign("NTE", "result")


class ScopioX100SegmentNTE(Segment):
    comment_source = HL7String(3)
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {"code": "scopio_x100", "label": "Scopio X100/X100HT"}
    SEGMENT_CLASS_MAP = {
        "MSH": ScopioX100SegmentMSH,
        "PID": ScopioX100SegmentPID,
        "PV1": ScopioX100SegmentPV1,
        "OBR": ScopioX100SegmentOBR,
        "OBX": ScopioX100SegmentOBX,
        "NTE": ScopioX100SegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_pv1 in seg_pid.visits:
                for seg_obr in seg_pv1.orders:
                    sample = worklist.get_or_create(
                        sample_id=seg_pid.patient_id, type="patient"
                    )
                    result_date = seg_obr.observation_date
                    for seg_obx in seg_obr.results:
                        comments = []
                        for seg_nte in seg_obx.comments:
                            comments.append(seg_nte.comment)
                        if seg_obx.value_type != "ED":
                            sample.results.add(
                                code=seg_obx.observation_identifier,
                                label=seg_obx.observation_text,
                                value=seg_obx.value,
                                unit=seg_obx.units,
                                operator="{} {}".format(
                                    seg_obx.operator_first_name,
                                    seg_obx.operator_last_name,
                                ),
                                date=result_date,
                                comments=comments,
                            )
                        else:
                            sample.results.add(
                                code=seg_obx.observation_identifier,
                                label=seg_obx.observation_text,
                                value=seg_obx.embended_file.line2,
                                operator="{} {}".format(
                                    seg_obx.operator_first_name,
                                    seg_obx.operator_last_name,
                                ),
                                date=result_date,
                                comments=comments,
                                graph=ScopioGraphContainer(
                                    data=base64.b64decode(seg_obx.embended_file.line5)
                                ),
                            )

        return worklist
