# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.fixed import FixedLengthFormat, Multiple, String
from driverng.drivers.tosoh.tosoh_utils import (
    get_sample_id,
    get_measurement_values,
    get_legacy_graphe,
    get_graphe,
    One,
    Many,
    ManyContainer,
)


class SampleInformation(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "1",
        # 0 = STD mode
        # 1 = VAR mode
        # 2 = beta-thalassemia mode
        String("operation_mode", 1, default="1"),
        # 0 = STAT
        # 1 = Transport (LA)
        # 2 = Loader
        String("sample_position", 1, default="1"),
        # /!\ Length 0, 13, 18 or 20
        String("sample_id", 20),
    ]


class SampleNumber(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "2",
        # /!\ Length 3, 5 or 8
        # The number of digits used for the sample number is set by the
        # SMPx key in the RS232C screen of the analyzer
        String("sample_number", 8),
    ]


# 3FL012345012345012345012345012345012345012345012345012345012345012345012345
# 324 17.63  0.77  0.70  0.70  4.75 16.16 80.24 30.50  0.00  0.00   153  524
# 300   0.0   0.0   0.3   1.1   0.3   2.4   6.0  91.2   0.0    42   0.0   469
# 300 11.42  0.62  0.68  0.75  1.79 10.12 88.52  0.00  0.00  0.00    87  567
# 307                                                                    178
# 307   0.0   1.2  54.9   3.5  18.2   0.0   0.0   0.0   0.0   0.0   0.0  1245


class MeasurementValue(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "3",
        # 00-99
        # 00 == Normal
        String("flag_judgement", 2, default="00"),
        # /!\ Length 5 or 6
        # Spaces or 0.00 ~ 100.00
        # String("total_a1_percent", 6, right_justified=True),
        Multiple(
            "components",
            [
                # Spaces or 0.00 ~ 100.00
                # Component | STD | VAR | Beta-thalassemia
                # 1 FP% A1a% (HbF%)
                # 2 A1a% A1b% (A0%)
                # 3 A1b% F% (HbA2%)
                # 4 F% LA1c% (HbD+%)
                # 5 LA1c% SA1c% or IFCC(*1) (HbS+%)
                # 6 SA1c% or IFCC(*1) A0% (HbC+%)
                # 7 A0% H-V0% (Reserved)
                # 8 (Reserved) H-V1% (Reserved)
                # 9 (Reserved) H-V2% (Reserved)
                # 10 Reserved or IFCC(*2) Reserved or IFCC(*2) (Reserved)
                # The number of digits used for each percent value obeys
                # (*2) PERCENT DECIMAL of the PARAMETER screen
                String("value", 6, right_justified=True)
            ],
        ),
        # 0 ~ 99999
        # TP ??
        String("reserved", 5, right_justified=True),
    ]


class DataCollectionParameters(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "4",
        # 0 ~ 9999 (x10mS)
        String("collection_start_time", 4, right_justified=True),
        # 0 ~ 9 (x100mS)
        String("collection_pitch", 1),
        # 0 ~ 20
        String("number_of_peaks", 2, right_justified=True),
        # 0 ~ 1000
        String("number_of_raw_data_sets", 4, right_justified=True),
    ]


class PeakData(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "5",
        # 1 ~ 20
        String("number", 2, right_justified=True),
        String("name", 5),
        # B = base peak
        # V = valley peak
        # S = skimming peak
        String("shape", 1),
        # 0 ~ 1000
        String("base_start_point", 4, right_justified=True),
        String("base_end_point", 4, right_justified=True),
        String("peak_start_point", 4, right_justified=True),
        String("peak_top", 4, right_justified=True),
        String("peak_end_point", 4, right_justified=True),
        # 0.00 ~ 99999.99 (mV sec)
        String("peak_area", 8, right_justified=True),
        # /!\ Length 5 or 6
        # 0.00 ~ 100.00
        String("peak_area_percent", 6, right_justified=True),
    ]


class PeakDataEnd(FixedLengthFormat):
    FMT_DESCRIPTION = ["6"]


class RawData(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "7",
        # 1 ~ 999
        String("block_number", 3, right_justified=True),
        Multiple(
            "block_signals",
            [
                # -9999.999 ~ 9999.999 (mV)
                String("value", 9, right_justified=True)
            ],
        ),
    ]


class CalibrationInformation(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "8",
        String("rack", 4),  # 0000 ~ 9999
        String("position", 2),  # 00 ~ 10
        # factor A of element 1
        # factor B of element 1
        # factor A of element 2
        # factor B of element 2
        Multiple("factors", [String("value", 8, right_justified=True)]),
    ]


class _TosohMessage(object):
    FMT_MAP = {
        "1": One("sample_information", SampleInformation),
        "2": One("sample_number", SampleNumber),
        "3": One("measurement_value", MeasurementValue),
        "4": One("data_collection_parameters", DataCollectionParameters),
        "5": Many("peak_datas", PeakData),
        "6": One("peak_data_end", PeakDataEnd),
        "7": Many("raw_datas", RawData),
        "8": One("calibration_information", CalibrationInformation),
    }

    def __init__(self, **kwargs):
        for attr in self.FMT_MAP.values():
            setattr(self, attr.name, attr.get())
        super(_TosohMessage, self).__init__()

    @staticmethod
    def parse(data):
        obj = _TosohMessage()
        for line in data.splitlines():
            if line:
                attr = obj.FMT_MAP[line[0]]
                value = getattr(obj, attr.name)
                if isinstance(value, ManyContainer):
                    o = value.add()
                    o.parse(line)
                else:
                    o = value.parse(line)

        return obj

    def to_string(self):
        lines = []
        for key in range(1, 9):
            attr = self.FMT_MAP[str(key)]
            value = getattr(self, attr.name)
            if isinstance(value, ManyContainer):
                lines.extend([v.to_string() for v in value])
            else:
                lines.append(value.to_string())
        return "\n".join(lines)


class Driver(MessageV2):
    SPEC = {"code": "tosoh", "label": "Tosoh G8"}
    COMPONENT_MAP = {
        "0": (
            "FP%",
            "A1a%",
            "A1b%",
            "F%",
            "LA1c%",
            "SA1c%",
            "A0%",
            "",
            "",
            "",
            "",
            "TP",
        ),
        "1": (
            "HbA1",
            "A1a%",
            "A1b%",
            "F%",
            "LA1c%",
            "SA1c%",
            "A0%",
            "H-V0%",
            "H-V1%",
            "H-V2%",
            "IFCC",
            "TP",
        ),
        "2": ("(HbF%)", "(A0%)", "(HbA2%)", "(HbD+%)", "(HbS+%)", "(HbC+%)"),
    }
    COMPONENT_MAP_CDS_G11 = {
        # Standard Analysis Mode
        "0": (
            "",
            "FP%",
            "A1a%",
            "A1b%",
            "F%",
            "LA1c%",
            "SA1c%",
            "A0%",
            "",
            "IFCC",
            "",
            "TP",
        ),
        # Variant Analysis Mode seul le format 1234567-S--- est utilisé par
        # TOSOH
        "1": (
            "",
            "FP%",
            "A1a%",
            "A1b%",
            "F%",
            "LA1c%",
            "SA1c%",
            "A0%",
            "",
            "IFCC",
            "",
            "TP",
        ),
        # Beta-Thalassemia Analysis Mode seul le format CDS FORM 1234567-----
        # est utilisé par TOSOH
        "2": ("", "F%", "A0%", "A2%", "E+%", "D+%", "S+%", "C+%", "", "", "", "TP"),
    }

    OPTIONS = [
        MessageV2.Option(
            "sample_id_mode",
            label="Gestion du code barre",
            choices=(
                ("normal", "Normal"),
                ("ignore_2", "Ignorer les deux premiers caractères"),
            ),
            default="ignore_2",
        ),
        MessageV2.Option(
            "version_tosoh",
            label="Version automate Tosoh",
            choices=(
                ("version_g8", "Tosoh version G8 Raw mode"),
                ("version_g11", "Tosoh version G11 CDS"),
                ("version_gx", "Tosoh version GX"),
            ),
            default="version_g8",
        ),
        MessageV2.BooleanOption(
            "tosoh_no_rack", label="Ne pas intégrer le rack du tube", default=False
        ),
        MessageV2.Option(
            "tosoh_qc_prefix",
            label="Liste des préfixes QC séparés par des ','",
            default="QC",
        ),
    ]

    def __init__(self, **kwargs):
        self.sample_id_mode = kwargs.pop("sample_id_mode", "ignore_2")
        self.version_tosoh = kwargs.pop("version_tosoh", "version_g8")
        self.tosoh_no_rack = kwargs.pop("tosoh_no_rack", False)
        self.betathal = False
        self.tosoh_qc_prefix = kwargs.pop("tosoh_qc_prefix", "QC")
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return _TosohMessage()

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message, crlf="\r\n"):
        return message.to_string()

    def parse(self, data):
        obj = _TosohMessage.parse(data)
        return obj

    def decode_message(self, message):

        worklist = Worklist(context="result")
        sample_id = get_sample_id(self, message)

        qc_prefix = [qc for qc in self.tosoh_qc_prefix.split(",") if qc]
        if sample_id.startswith(tuple(qc_prefix)) or sample_id.endswith(
            tuple(qc_prefix)
        ):
            sample = worklist.get_or_create(
                type="control", control_lot=sample_id, sample_id=sample_id
            )

            for test, value in get_measurement_values(self, message):
                sample.results.add(code=test, value=value)
            for peak in message.peak_datas:
                if peak.name == "SA1C":
                    sample.results.add(
                        code="areap_{:s}".format(peak.name),
                        value="{:.2f}".format(float(peak.peak_area_percent)),
                    )
            return worklist

        if self.tosoh_no_rack:
            rack = ""
        else:
            rack = "{:s}-{:s}".format(
                message.calibration_information.rack,
                message.calibration_information.position,
            )
        sample = worklist.get_or_create(
            type="patient",
            sample_id=sample_id,
            rack=rack,
            request_mode_hbdrive=message.sample_information.operation_mode,
        )

        sample.results.add(code="RACK", value=message.sample_number.sample_number)

        if message.measurement_value and not self.betathal:
            flags = message.measurement_value.flag_judgement
        elif message.measurement_value_betathal:
            flags = message.measurement_value_betathal.flag_judgement
        else:
            flags = None

            # Pas de soucis
        if flags == "00":
            flags = []

        else:  # ICI, flags peut etre vide (si absence de measurement_value)
            sample.results.add(code="HB_ERR", value=flags if flags else ["HB_ERR"])
            flags = [flags]

        if (
            message.calibration_information.factors
            and len(message.calibration_information.factors) >= 2
        ):
            sample.results.add(
                code="FACTEURA", value=message.calibration_information.factors[0].value
            )
            sample.results.add(
                code="FACTEURB", value=message.calibration_information.factors[1].value
            )
            if len(message.calibration_information.factors) >= 3:
                sample.results.add(
                    code="CALIBRATION_RESERVED1",
                    value=message.calibration_information.factors[2].value,
                )
            if len(message.calibration_information.factors) >= 4:
                sample.results.add(
                    code="CALIBRATION_RESERVED2",
                    value=message.calibration_information.factors[3].value,
                )

        for test, value in get_measurement_values(self, message):
            sample.results.add(code=test, value=value, flags=flags)

        for g in get_legacy_graphe(self, message, flags) + get_graphe(self, message):
            sample.results.add(g)

        return worklist
